﻿
using System;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

using BingPaper.Interfaces;
using BingPaper.PInvoke;
using BingPaper.Util;
using Microsoft.Win32;
using Debug = System.Diagnostics.Debug;
using BingPaper.Impl.Command;

namespace BingPaper.Impl
{
	internal class AppCore : IAppCore
	{
		private ILogger _logger;
		private IConfigStorage _storage;
		
		private bool _isRunning = false;
		private Config _config;
		
		private Timer _timer;
		
		private string _preImageName;
		
		private bool _isIdle = true;
		
		private const string RUN_ERROR = "Cannot change properties when running";
		private const long SECOND = 1000L;
		
		public AppCore()
		{
			 _timer = new Timer(TimerCallback, null, Timeout.Infinite, Timeout.Infinite);
		}
		
		public ILogger Logger {
			get { return _logger; }
			set {
				ValidateStateAndValue(value);
				_logger = value;
			}
		}
		
		public IConfigStorage ConfigStorage {
			get { return _storage; }
			set {
				ValidateStateAndValue(value);
				_storage = value;
			}
		}
		
		public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;
		
		public bool IsIdle {
			get {
				return _isIdle;
			}
			private set{
				_isIdle = value;
				OnPropertyChanged(new PropertyChangedEventArgs("IsIdle"));
			}
		}

		private void ValidateStateAndValue(object value)
		{
			if (value == null)
				throw new ArgumentNullException("value");
			if (_isRunning)
				throw new InvalidOperationException(RUN_ERROR);
		}
		
		public void Run()
		{
			var config = ConfigStorage.LoadConfig();
			var err = config.Validate();
			if (err.Any())
			{
				foreach (var item in err) 
					_logger.LogError(item.Value);
				return;
			}
			_config = config;
			_isRunning = true;
			_timer.Change(0, Timeout.Infinite);
		}
		
		public void Stop()
		{
			_timer.Change(Timeout.Infinite, Timeout.Infinite); //stop the timer
			_isRunning = false;
			_config = null;
		}
		
		public string SavedImagePath {
			get {
                return Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures), "bing_save.bmp");
			}
		}
		
		public void NotifyWallpaperChanged()
		{
			if (!_isRunning || !_config.PreventBlackWallpaper)
				return;
			string desktopWallpaper = GetDesktopWallpaperPath(_logger);
			if (string.IsNullOrWhiteSpace(desktopWallpaper))
			{
				Logger.LogMessage("No wallpaper applied, use Bing image.");
                if (File.Exists(SavedImagePath))
                    CommandBuilder.GenereateSetWallpaperCommand().Run(new CommandContext(this));
                else
                    _timer.Change(0, Timeout.Infinite); //get image and applied
			}
		}
		
		protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
		{
			PropertyChangedEventHandler propertyChangedEventHandler = Interlocked.CompareExchange<PropertyChangedEventHandler>(ref this.PropertyChanged, null, null);
			if (propertyChangedEventHandler != null)
			{
				propertyChangedEventHandler(this, e);
			}
		}
		
		private void TimerCallback(object state)
		{
			IsIdle = false;
			try {
                CommandBuilder.GenerateUpateWallpaperCommand().Run(new CommandContext(this));
			}catch(Exception ex){
				_logger.LogError(ex.Message);
			}finally{
				long periold = ComputePeriod();
				_logger.LogMessage(string.Format("Download Bing image after {0} seconds", periold / SECOND));
				_timer.Change(periold, Timeout.Infinite);
				IsIdle = true;
			}
		}
		
		private long ComputePeriod()
		{
			if (_config.PeriodType == Config.PERIOD_MINUTE)
				return long.Parse(_config.Period) * 60 * SECOND;
			else if(_config.PeriodType == Config.PERIOD_TIME){
				DateTime now = DateTime.Now;
				string[] pair = _config.Period.Split(':');
				Debug.Assert(pair.Length == 2);
				DateTime toDate = new DateTime(now.Year, now.Month, now.Day, int.Parse(pair[0]), int.Parse(pair[1]), 0);
				TimeSpan span = toDate - now;
				if (span.TotalMilliseconds < 0)
				{
					toDate = toDate.AddDays(1);
					span = toDate - now;
					Debug.Assert(span.TotalMilliseconds > 0);
				}
				return (long)span.TotalMilliseconds;
			} else {
				_logger.LogError("Unknown Period Type");
				return 0L;
			}
		}

        private static string GetDesktopWallpaperPath(ILogger logger)
        {
            StringBuilder deskBackgroundPath = new StringBuilder(256);
            if (!SPIWrapper.SystemParametersInfo(SPI.SPI_GETDESKWALLPAPER, 256u, deskBackgroundPath, SPIF.None))
            {
                logger.LogError(string.Format("Get desktop wallpaper failed : 0x{0:8x}", Marshal.GetLastWin32Error()));
                return string.Empty;
            }
            logger.LogMessage(string.Format("Current wallpaper is : {0}", deskBackgroundPath.ToString()));
            return deskBackgroundPath.ToString();
        }

        private class CommandContext : ICommandContext
        {
            private AppCore _core;
            private string _currentImageName;

            public CommandContext(AppCore core)
            {
                _core = core;
                _currentImageName = core._preImageName;
                Logger = core.Logger;
                ImageSavePath = core.SavedImagePath;
                Config = core._config;
            }

            public ILogger Logger { get; set; }

            public string ImageSavePath { get; set; }

            public Config Config {get; set;}

            public string CurrentImageName
            {
                get
                {
                    return _currentImageName;
                }
                set
                {
                    if (!string.IsNullOrWhiteSpace(value))
                        _core._preImageName = value;
                    _currentImageName = value;
                }
            }
        }
	}
}
