﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Linq.Expressions;
using System.Windows.Forms;

using BingPaper.Interfaces;
using BingPaper.Util;
using Debug = System.Diagnostics.Debug;

namespace BingPaper.UI
{
	internal partial class SettingForm : Form
	{
		private SettingModel _model;
		
		public SettingForm(Config config)
		{
			InitializeComponent();
			
			_model = new SettingForm.SettingModel(config);
			
			lstPriodType.DisplayMember = "Text";
			lstPriodType.ValueMember = "Value";
			lstPriodType.DataSource = _model.PeriodTypeList.ToList();
			lstPriodType.DataBindings.Add("SelectedValue", _model, "PeriodType", false, DataSourceUpdateMode.OnPropertyChanged);
			txtPriod.DataBindings.Add("Text", _model, "Period", false, DataSourceUpdateMode.OnPropertyChanged);
			chkPreventBlackScreen.DataBindings.Add("Checked", _model, "PreventBlackScreen", false, DataSourceUpdateMode.OnPropertyChanged);
			lstCountry.DisplayMember = "Name";
			lstCountry.ValueMember = "Code";
			lstCountry.DataSource = CountryListProvider.BingCountries.ToList();
			lstCountry.DataBindings.Add("SelectedValue", _model, "Country", false, DataSourceUpdateMode.OnPropertyChanged);
			lstCountry.DataBindings.Add("Enabled", _model, "EnableCountryList", false, DataSourceUpdateMode.OnPropertyChanged);
			chkRandomCountry.DataBindings.Add("Checked", _model, "RandomCountry", false, DataSourceUpdateMode.OnPropertyChanged);
			lstWpStyle.DataSource = (from x in WallPaperStyleListProvider.StyleNameValues
			                         select new TextIntItem {Text = x.Key, Value = x.Value}).ToList();
			lstWpStyle.DisplayMember = "Text";
			lstWpStyle.ValueMember = "Value";
			lstWpStyle.DataBindings.Add("SelectedValue", _model, "WallpaperStyle", false, DataSourceUpdateMode.OnPropertyChanged);
			chkAutoGlassColor.DataBindings.Add("Checked", _model, "AutoWindowColor", false, DataSourceUpdateMode.OnPropertyChanged);
			
			
			errProvider.DataSource = _model;
		}
		
		public Config Config
		{
			get {return _model.CreateConfig();}
		}
		
		protected override void OnFormClosing(FormClosingEventArgs e)
		{
			base.OnFormClosing(e);
			if (this.DialogResult == DialogResult.OK){
				if (_model.CreateConfig().Validate().Any())
					e.Cancel = true;
			}
		}
		
		private class SettingModel : IDataErrorInfo, INotifyPropertyChanged
		{	
			private int _periodType;
			private string _period;
			private bool _preventBlackScreen;
			private bool _randomCountry;
			private string _country;
			private int _wallpaperStyle;
			private bool _autoWindowColor;
			
			public SettingModel(Config config)
			{
				_periodType = config.PeriodType;
				_period = config.Period;
				_preventBlackScreen = config.PreventBlackWallpaper;
				_country = config.CountryCode;
				_randomCountry = config.RandomCountryCode;
				_wallpaperStyle = config.WallPaperStyle;
				_autoWindowColor = config.AutoWindowColor;
			}
			
			public bool AutoWindowColor {
				get { return _autoWindowColor; }
				set { 
					if (_autoWindowColor != value)
					{
						_autoWindowColor = value;
						RaisePropertyChange(() => this.AutoWindowColor);
					}
				}
			}
						
			public int WallpaperStyle {
				get { return _wallpaperStyle; }
				set { 
					if (_wallpaperStyle != value)
					{
						_wallpaperStyle = value;
						RaisePropertyChange(() => this.WallpaperStyle);
					}
				}
			}
			
			public bool EnableCountryList {
				get { return !_randomCountry; }
			}
			
			public bool RandomCountry {
				get { return _randomCountry; }
				set { 
					if (_randomCountry != value){
						_randomCountry = value;
						RaisePropertyChange(() => this.RandomCountry);
						RaisePropertyChange(() => this.EnableCountryList);
					}
				}
			}
			
			public string Country {
				get { return _country; }
				set { 
					if (!string.Equals(_country, value, StringComparison.Ordinal))
					{
						_country = value;
						RaisePropertyChange(() => this.Country);
					}
				}
			}
			
			public int PeriodType {
				get {return _periodType;}
				set {
					if (_periodType != value)
					{
						_periodType = value;
						RaisePropertyChange(() => this.PeriodType);
					}
				}
			}
			
			public string Period {
				get {return _period;}
				set {
					if (!string.Equals(_period, value, StringComparison.Ordinal))
					{
						_period = value;
						RaisePropertyChange(() => this.Period);
					}
				}
			}
			
			public bool PreventBlackScreen {
				get { return _preventBlackScreen;}
				set {
					if (_preventBlackScreen != value)
					{
						_preventBlackScreen = value;
						RaisePropertyChange(() => this.PreventBlackScreen);
					}
				}
			}
			
			public string this[string item] {
				get {
					var err = CreateConfig().Validate();
					var msg = from x in err where string.Equals(x.Key, item, StringComparison.Ordinal)
						select x.Value;
					if (msg.Any())
						return msg.First();
					return string.Empty;
				}
			}
			
			public string Error {
				get {
					var err = CreateConfig().Validate();
					if (err.Any())
						return err.First().Value;
					return string.Empty;
				}
			}
			
			public Config CreateConfig()
			{
				return new Config
				{
					PeriodType = PeriodType,
					Period = Period,
					PreventBlackWallpaper = PreventBlackScreen,
					CountryCode = Country,
					RandomCountryCode = RandomCountry,
					WallPaperStyle = WallpaperStyle,
					AutoWindowColor = AutoWindowColor
				};
			}
			
			public IEnumerable<TextIntItem> PeriodTypeList
			{
				get {
					yield return new TextIntItem {Text = "Minute", Value = Config.PERIOD_MINUTE};
					yield return new TextIntItem {Text = "Time", Value = Config.PERIOD_TIME};
				}
			}
			
			public event PropertyChangedEventHandler PropertyChanged;
			
			protected void OnPropertyChanged(PropertyChangedEventArgs e)
			{
				PropertyChangedEventHandler handler = System.Threading.Interlocked.CompareExchange(ref PropertyChanged, null, null);
				if (handler != null)
					handler(this, e);
			}
			
			private void RaisePropertyChange<T>(Expression<Func<T>> express)
			{
				MemberExpression memberExp = express.Body as MemberExpression;
				Debug.Assert(memberExp != null);
				OnPropertyChanged(new PropertyChangedEventArgs(memberExp.Member.Name));
			}
		}
		
		private class TextIntItem 
		{
			public string Text {get; set;}
			public int Value {get; set;}
		}
	}
}
