﻿
using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace BingPaper.Util
{
	internal static class WindowColorCalculator
	{
        private delegate void ColorCallback(int x, int y, out byte r, out byte g, out byte b);

        private interface ICalculator
        {
            Color Calculate(ColorCallback callback, int width, int height);
        }

        private class ColorRetriver {
            BitmapData _data;
            public ColorRetriver(BitmapData data)
            {
                _data = data;
            }

            public unsafe void GetColor(int x, int y, out byte r, out byte g, out byte b)
            {
                byte* pixelPtr = (byte*)_data.Scan0;
                pixelPtr += _data.Stride * y + x;
                b = *pixelPtr;
                g = *(pixelPtr + 1);
                r = *(pixelPtr + 2);
            }
        }

		public static Color Calculate(Image image)
        {
            if (image.PixelFormat != PixelFormat.Format24bppRgb)
            {
                Console.Error.WriteLine("Only 24bit color supported");
                return Color.Black;
            }
            
            Bitmap bitmap = image as Bitmap;
            if (image == null)
            {
            	Console.Error.WriteLine("Not a bitmap.");
                return Color.Black;
            }

            return Calculate(bitmap, new PickCenterCalculator(50));
        }

        private static Color Calculate(Bitmap image, ICalculator calculator)
        {
            var bitmapData = image.LockBits(new Rectangle(0, 0, image.Width, image.Height), ImageLockMode.ReadOnly, image.PixelFormat);
            try {
                return calculator.Calculate((new ColorRetriver(bitmapData)).GetColor, image.Width, image.Height);
            }
            finally {
                image.UnlockBits(bitmapData);
            }
        }

        private class AverageCalculator : ICalculator
        {
            public Color Calculate(ColorCallback callback, int width, int height)
            {
                UInt64 r = 0, g = 0, b = 0;
                byte br, bg, bb;
                for(int y = 0; y < height; ++y)
                    for (int x = 0; x < width; ++x)
                    {
                        callback(x, y, out br, out bg, out bb);
                        r += br;
                        g += bg;
                        b += bb;
                    }
                UInt64 count = (UInt64)(width * height);
                return Color.FromArgb((int)(r / count), (int)(g / count), (int)(b / count));
            }
        }

        private class PickCenterCalculator : ICalculator
        {
            int _sideLength;
            public PickCenterCalculator() : this(10) {
            }

            public PickCenterCalculator(int sideLength)
            {
                if (sideLength <= 0)
                    sideLength = 10;
                _sideLength = sideLength;
            }

            public Color Calculate(ColorCallback callback, int width, int height)
            {
                int startX = (width - _sideLength) / 2;
                int endX = startX + _sideLength;
                startX = startX > 0 ? startX : 0;
                endX = endX < width ? endX : width - 1;

                int startY = (height - _sideLength) / 2;
                int endY = startY + _sideLength;
                startY = startY > 0 ? startY : 0;
                endY = endY < height ? endY : height - 1;

                UInt64 count = (UInt64)((endX - startX + 1) * (endY - startY + 1));

                UInt64 r = 0, g = 0, b = 0;
                byte br, bg, bb;
                for (int y = startY; y <= endY; ++y)
                    for (int x = startX; x <= endX; ++x)
                    {
                        callback(x, y, out br, out bg, out bb);
                        r += br;
                        g += bg;
                        b += bb;
                    }
                return Color.FromArgb((int)(r / count), (int)(g / count), (int)(b / count));
            }
        }
	}
}
